==> open-xchange-core_7.2.2.orig.tar.bz2: open-xchange-core-7.2.2/com.openexchange.server/src/com/openexchange/groupware/tasks/InsertData.java <==
/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks of the Open-Xchange, Inc. group of companies.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2004-2013 Open-Xchange, Inc.
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.openexchange.groupware.tasks;

import static com.openexchange.groupware.tasks.StorageType.ACTIVE;
import static com.openexchange.java.Autoboxing.I;
import static com.openexchange.tools.sql.DBUtils.autocommit;
import static com.openexchange.tools.sql.DBUtils.rollback;
import java.sql.Connection;
import java.sql.SQLException;
import java.util.Set;
import java.util.concurrent.atomic.AtomicReference;
import com.openexchange.event.impl.EventClient;
import com.openexchange.exception.OXException;
import com.openexchange.groupware.Types;
import com.openexchange.groupware.container.FolderObject;
import com.openexchange.groupware.contexts.Context;
import com.openexchange.groupware.impl.IDGenerator;
import com.openexchange.groupware.ldap.User;
import com.openexchange.groupware.userconfiguration.UserConfiguration;
import com.openexchange.quota.Quota;
import com.openexchange.quota.QuotaExceptionCodes;
import com.openexchange.quota.QuotaService;
import com.openexchange.quota.QuotaType;
import com.openexchange.quota.Resource;
import com.openexchange.quota.ResourceDescription;
import com.openexchange.server.impl.DBPool;
import com.openexchange.session.Session;

/**
 * Implements the insertion of a task.
 *
 * @author <a href="mailto:marcus.klein@open-xchange.com">Marcus Klein</a>
 */
public final class InsertData {

    private static final AtomicReference<QuotaService> QUOTA_SERVICE_REF = new AtomicReference<QuotaService>();

    private final Context ctx;
    private final User user;
    private final UserConfiguration userConfig;
    private final FolderObject folder;
    private final Task task;

    private Set<TaskParticipant> parts;
    private Set<Folder> folders;

    InsertData(Context ctx, User user, UserConfiguration userConfig, FolderObject folder, Task task) {
        super();
        this.ctx = ctx;
        this.user = user;
        this.userConfig = userConfig;
        this.folder = folder;
        this.task = task;
    }

    /**
     * Sets {@link QuotaService} reference to given instance.
     * 
     * @param quotaService The {@link QuotaService} instance or <code>null</code>
     */
    public static void setQuotaService(QuotaService quotaService) {
        QUOTA_SERVICE_REF.set(quotaService);
    }

    void prepare(Session session) throws OXException {
        parts = TaskLogic.createParticipants(ctx, task.getParticipants());
        TaskLogic.checkNewTask(task, user.getId(), userConfig, parts);

        // Check access rights
        Permission.checkCreate(ctx, user, userConfig, folder);
        int folderId = folder.getObjectID();
        if (task.getPrivateFlag() && (Tools.isFolderPublic(folder) || Tools.isFolderShared(folder, user))) {
            throw TaskExceptionCode.PRIVATE_FLAG.create(I(folderId));
        }

        // Create folder mappings
        if (Tools.isFolderPublic(folder)) {
            folders = TaskLogic.createFolderMapping(folderId, task.getCreatedBy(), InternalParticipant.EMPTY_INTERNAL);
        } else {
            Tools.fillStandardFolders(ctx, ParticipantStorage.extractInternal(parts));
            int creator = user.getId();
            if (Tools.isFolderShared(folder, user)) {
                creator = folder.getCreator();
            }
            folders = TaskLogic.createFolderMapping(folderId, creator,ParticipantStorage.extractInternal(parts));
        }

        // Check if over quota
        QuotaService quotaService = QUOTA_SERVICE_REF.get();
        if (null != quotaService) {
            Quota quota = quotaService.getQuotaFor(Resource.TASK, ResourceDescription.getEmptyResourceDescription(), session);
            long amount = quota.getQuota(QuotaType.AMOUNT);
            if (amount > 0) {
                int numberOfTasks = storage.countTasks(ctx);
                if (numberOfTasks + 1 > amount) {
                    throw QuotaExceptionCodes.QUOTA_EXCEEDED.create();
                }
            }
        }
    }

    void doInsert() throws OXException {
        insertTask(ctx, task, parts, folders);
    }

    void createReminder() throws OXException {
        if (task.containsAlarm()) {
            Reminder.createReminder(ctx, task);
        }
    }

    void sentEvent(Session session) throws OXException {
        // Prepare for event
        task.setUsers(TaskLogic.createUserParticipants(parts));
        new EventClient(session).create(task, folder);
    }

    private static final TaskStorage storage = TaskStorage.getInstance();
    private static final ParticipantStorage partStor = ParticipantStorage.getInstance();
    private static final FolderStorage foldStor = FolderStorage.getInstance();

    /**
     * Stores a task with its participants and folders.
     *
     * @param ctx Context.
     * @param task Task to store.
     * @param participants Participants of the task.
     * @param folders Folders the task should appear in.
     * @throws OXException if an error occurs while storing the task.
     */
    static void insertTask(Context ctx, Task task, Set<TaskParticipant> participants, Set<Folder> folders) throws OXException {
        final Connection con = DBPool.pickupWriteable(ctx);
        try {
            con.setAutoCommit(false);
            final int taskId = IDGenerator.getId(ctx, Types.TASK, con);
            task.setObjectID(taskId);
            storage.insertTask(ctx, con, task, ACTIVE);
            if (participants.size() != 0) {
                partStor.insertParticipants(ctx, con, taskId, participants, ACTIVE);
            }
            foldStor.insertFolder(ctx, con, taskId, folders, ACTIVE);
            con.commit();
        } catch (SQLException e) {
            rollback(con);
            throw TaskExceptionCode.INSERT_FAILED.create(e, e.getMessage());
        } catch (OXException e) {
            rollback(con);
            throw e;
        } finally {
            autocommit(con);
            DBPool.closeWriterSilent(ctx, con);
        }
    }

}

==> open-xchange-core_7.4.0.orig.tar.bz2: open-xchange-core-7.4.0/com.openexchange.server/src/com/openexchange/groupware/tasks/InsertData.java <==
/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks of the Open-Xchange, Inc. group of companies.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2004-2013 Open-Xchange, Inc.
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.openexchange.groupware.tasks;

import static com.openexchange.groupware.tasks.StorageType.ACTIVE;
import static com.openexchange.java.Autoboxing.I;
import static com.openexchange.tools.sql.DBUtils.autocommit;
import static com.openexchange.tools.sql.DBUtils.rollback;
import java.sql.Connection;
import java.sql.SQLException;
import java.util.Set;
import java.util.concurrent.atomic.AtomicReference;
import com.openexchange.event.impl.EventClient;
import com.openexchange.exception.OXException;
import com.openexchange.groupware.Types;
import com.openexchange.groupware.container.FolderObject;
import com.openexchange.groupware.contexts.Context;
import com.openexchange.groupware.impl.IDGenerator;
import com.openexchange.groupware.ldap.User;
import com.openexchange.groupware.userconfiguration.UserPermissionBits;
import com.openexchange.quota.Quota;
import com.openexchange.quota.QuotaExceptionCodes;
import com.openexchange.quota.QuotaService;
import com.openexchange.quota.QuotaType;
import com.openexchange.quota.Resource;
import com.openexchange.quota.ResourceDescription;
import com.openexchange.server.impl.DBPool;
import com.openexchange.session.Session;

/**
 * Implements the insertion of a task.
 *
 * @author <a href="mailto:marcus.klein@open-xchange.com">Marcus Klein</a>
 */
public final class InsertData {

    private static final AtomicReference<QuotaService> QUOTA_SERVICE_REF = new AtomicReference<QuotaService>();

    private final Context ctx;
    private final User user;
    private final UserPermissionBits userConfig;
    private final FolderObject folder;
    private final Task task;

    private Set<TaskParticipant> parts;
    private Set<Folder> folders;

    InsertData(Context ctx, User user, UserPermissionBits userConfig, FolderObject folder, Task task) {
        super();
        this.ctx = ctx;
        this.user = user;
        this.userConfig = userConfig;
        this.folder = folder;
        this.task = task;
    }

    /**
     * Sets {@link QuotaService} reference to given instance.
     *
     * @param quotaService The {@link QuotaService} instance or <code>null</code>
     */
    public static void setQuotaService(QuotaService quotaService) {
        QUOTA_SERVICE_REF.set(quotaService);
    }

    void prepare(Session session) throws OXException {
        parts = TaskLogic.createParticipants(ctx, task.getParticipants());
        TaskLogic.checkNewTask(task, user.getId(), userConfig, parts);

        // Check access rights
        Permission.checkCreate(ctx, user, userConfig, folder);
        int folderId = folder.getObjectID();
        if (task.getPrivateFlag() && (Tools.isFolderPublic(folder) || Tools.isFolderShared(folder, user))) {
            throw TaskExceptionCode.PRIVATE_FLAG.create(I(folderId));
        }

        // Create folder mappings
        if (Tools.isFolderPublic(folder)) {
            folders = TaskLogic.createFolderMapping(folderId, task.getCreatedBy(), InternalParticipant.EMPTY_INTERNAL);
        } else {
            Tools.fillStandardFolders(ctx, ParticipantStorage.extractInternal(parts));
            int creator = user.getId();
            if (Tools.isFolderShared(folder, user)) {
                creator = folder.getCreator();
            }
            folders = TaskLogic.createFolderMapping(folderId, creator,ParticipantStorage.extractInternal(parts));
        }

        // Check if over quota
        QuotaService quotaService = QUOTA_SERVICE_REF.get();
        if (null != quotaService) {
            Quota quota = quotaService.getQuotaFor(Resource.TASK, ResourceDescription.getEmptyResourceDescription(), session);
            long amount = quota.getQuota(QuotaType.AMOUNT);
            if (amount > 0) {
                int numberOfTasks = storage.countTasks(ctx);
                if (numberOfTasks + 1 > amount) {
                    throw QuotaExceptionCodes.QUOTA_EXCEEDED.create();
                }
            }
        }
    }

    void doInsert() throws OXException {
        insertTask(ctx, task, parts, folders);
    }

    void createReminder() throws OXException {
        if (task.containsAlarm()) {
            Reminder.createReminder(ctx, task);
        }
    }

    void sentEvent(Session session) throws OXException {
        // Prepare for event
        task.setUsers(TaskLogic.createUserParticipants(parts));
        new EventClient(session).create(task, folder);
    }

    private static final TaskStorage storage = TaskStorage.getInstance();
    private static final ParticipantStorage partStor = ParticipantStorage.getInstance();
    private static final FolderStorage foldStor = FolderStorage.getInstance();

    /**
     * Stores a task with its participants and folders.
     *
     * @param ctx Context.
     * @param task Task to store.
     * @param participants Participants of the task.
     * @param folders Folders the task should appear in.
     * @throws OXException if an error occurs while storing the task.
     */
    static void insertTask(Context ctx, Task task, Set<TaskParticipant> participants, Set<Folder> folders) throws OXException {
        final Connection con = DBPool.pickupWriteable(ctx);
        try {
            con.setAutoCommit(false);
            final int taskId = IDGenerator.getId(ctx, Types.TASK, con);
            task.setObjectID(taskId);
            storage.insertTask(ctx, con, task, ACTIVE);
            if (participants.size() != 0) {
                partStor.insertParticipants(ctx, con, taskId, participants, ACTIVE);
            }
            foldStor.insertFolder(ctx, con, taskId, folders, ACTIVE);
            con.commit();
        } catch (SQLException e) {
            rollback(con);
            throw TaskExceptionCode.INSERT_FAILED.create(e, e.getMessage());
        } catch (OXException e) {
            rollback(con);
            throw e;
        } finally {
            autocommit(con);
            DBPool.closeWriterSilent(ctx, con);
        }
    }

}
